package cn.hanghui.outapi.platform.common.utils;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.HexUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.crypto.asymmetric.KeyType;
import cn.hutool.crypto.asymmetric.SM2;
import org.bouncycastle.jce.interfaces.ECPrivateKey;
import org.bouncycastle.jce.interfaces.ECPublicKey;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * @author qhdz
 * @description: SM2秘钥生成、签名、验签工具类
 * @date 2024/4/27-下午7:57
 */
public class Sm2Util {

    /**
     * 公钥常量
     */
    public static final String KEY_PUBLIC_KEY = "publicKey";
    /**
     * 私钥返回值常量
     */
    public static final String KEY_PRIVATE_KEY = "privateKey";

    public static final int NUM_2 = 2;
    public static final int NUM_16 = 16;
    public static final int NUM_64 = 64;

    /**
     * 生成SM2公私钥
     *
     * @return
     */
    public static Map<String, String> generateSm2Key() {
        SM2 sm2 = new SM2();
        ECPublicKey publicKey = (ECPublicKey) sm2.getPublicKey();
        ECPrivateKey privateKey = (ECPrivateKey) sm2.getPrivateKey();
        // 获取公钥
        byte[] publicKeyBytes = publicKey.getQ().getEncoded(false);
        String publicKeyHex = HexUtil.encodeHexStr(publicKeyBytes);

        // 获取64位私钥
        String privateKeyHex = privateKey.getD().toString(NUM_16);
        // BigInteger转成16进制时，不一定长度为64，如果私钥长度小于64，则在前方补0
        StringBuilder privateKey64 = new StringBuilder(privateKeyHex);
        while (privateKey64.length() <NUM_64) {
            privateKey64.insert(0, "0");
        }

        Map<String, String> result = new HashMap<>(NUM_2);
        result.put(KEY_PUBLIC_KEY, publicKeyHex);
        result.put(KEY_PRIVATE_KEY, privateKey64.toString());
        return result;
    }

    /**
     * SM2公钥加密
     *
     * @param content   原文
     * @param publicKey SM2公钥
     * @return
     */
    public static String encryptBase64(String content, String publicKey) {
        SM2 sm2 = new SM2(null, publicKey);
        return HexUtil.encodeHexStr(sm2.encrypt(content, KeyType.PublicKey)) ;
    }

    /**
     * SM2私钥解密
     *
     * @param encryptStr SM2加密字符串
     * @param privateKey SM2私钥
     * @return
     */
    public static String decryptBase64(String encryptStr, String privateKey) {
        SM2 sm2 = new SM2(privateKey, null);
        return StrUtil.utf8Str(sm2.decrypt(encryptStr, KeyType.PrivateKey));
    }

    /**
     * SM2私钥签名
     *
     * @param privateKey 私钥
     * @param content    待签名内容
     * @return 签名值
     */
    public static String sign(String privateKey, String content) {
        SM2 sm2 = new SM2(privateKey, null);
        return sm2.signHex(HexUtil.encodeHexStr(content));
    }

    /**
     * SM2公钥验签
     *
     * @param publicKey 公钥
     * @param content   原始内容
     * @param sign      签名
     * @return 验签结果
     */
    public static boolean verify(String publicKey, String content, String sign) {
        SM2 sm2 = new SM2(null, publicKey);
        return sm2.verifyHex(HexUtil.encodeHexStr(content), sign);
    }


    public static void main(String[] args) {
        Map<String, String> sm2Key = Sm2Util.generateSm2Key();
        String privateKey = sm2Key.get(KEY_PRIVATE_KEY);
        String publicKey = sm2Key.get(KEY_PUBLIC_KEY);
        System.out.println("公钥"+privateKey);
        System.out.println("私钥"+publicKey);
        Date nowDate = new Date();
        String time = DateUtil.format(nowDate, "yyyyMMddHHmmss");
        System.out.println("time："+time);
        String testStr = "hello国密" ;
        String mw=encryptBase64(testStr,publicKey);
        System.out.println("加密密文：" + mw);
        String mwss=decryptBase64(mw,privateKey);
        System.out.println("解密密文：" + mwss);

        // 使用私钥SM2签名
        String sign = Sm2Util.sign(privateKey, testStr);
        System.out.println("签名值：" + sign);

        // 私钥公钥验签成功例子
        boolean verify = Sm2Util.verify(publicKey, testStr, sign);
        System.out.println("正确的验签结果：" + verify);
        // 私钥公钥验签失败例子
        System.out.println("内容改变，验签结果：" + Sm2Util.verify(publicKey, "error", sign));
        System.out.println("公钥错误，验签结果：" + Sm2Util.verify(Sm2Util.generateSm2Key().get(Sm2Util.KEY_PUBLIC_KEY), testStr, sign));
        String errorSign = "3046022100e39b3937056d9652c10d3aa8a850b8b4075ec7c0adea60af26e4865121d400ad022100d41295331c29e14bd5e70f7325a3d9ac1716662cc766db710323dac873d62ab0";
        System.out.println("错误签名，验签结果：" + Sm2Util.verify(publicKey, testStr, errorSign));

    }

}
